﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using EventStore.ClientAPI;
using EventStore.ClientAPI.SystemData;
using Timer = System.Timers.Timer;

namespace ES_TestProject
{
    class Program
    {
        private const string StreamId = "TestStream";
        private static Timer _timer;
        private static EventStoreAllCatchUpSubscription _subscription;
        private static Position? _startPosition;
        private static int _lastEventNumber;
        private static int _count = 0;

        static void Main(string[] args)
        {
            var es = new ESConnection().Connection;
            var ts = new TimeSpan(0, 0, 0, 4);

            var md = StreamMetadata.Create(null, ts);

            Console.WriteLine("Setting MetaData And Starting Subscription");
            _subscription = es.SubscribeToAllFrom(Position.End, true, WhenEventAppeared, null, null);
            var task = es.SetStreamMetadataAsync(StreamId, StreamExpectedVersion.Any, md);
            task.Wait();


            Console.WriteLine("Stating Timer");
            _timer = new Timer()
            {
                AutoReset = true,
                Enabled = true,
                Interval = 1000
            };
            _timer.Elapsed += (sender, timerArgs) => TriggerExecution(es);
            _timer.Start();

            Console.ReadLine();
        }

        static void WhenEventAppeared(EventStoreCatchUpSubscription arg1, ResolvedEvent evt)
        {
            if (!evt.OriginalStreamId.StartsWith("$") && !evt.OriginalStreamId.StartsWith("@"))
            {
                Console.WriteLine("Event Appeared at {0}", evt.OriginalEventNumber);
                _startPosition = evt.OriginalPosition;
                _lastEventNumber = evt.OriginalEventNumber;
            }
        }

        static void TriggerExecution(IEventStoreConnection connection)
        {
            var evt = new TestEvent();
            var data = SerializationExtensions.Serialize(evt);
            var ed = new EventData(Guid.NewGuid(), "TestEvent", true, data, null);

            var task = connection.AppendToStreamAsync(StreamId, StreamExpectedVersion.Any, new[] { ed });

            _count++;

            if (_count == 12)
            {
                _count = 0;
                _timer.Stop();
                _subscription.Stop();

                Console.WriteLine("Sleeping.......");
                Thread.Sleep(5000);

                _subscription = connection.SubscribeToAllFrom(Position.End, true, WhenEventAppeared, null, null);
                _timer.Start();
                Console.WriteLine("Timer & SubScription Restarted");
            }
        }

    }

    public class TestEvent
    {

    }

    public class StreamExpectedVersion
    {
        public const int Any = -2;
        public const int NoStream = -1;
        public const int Invalid = -3;
    }
}
